--*******************************************************************************************************************************
--
-- Description: This module creates a sample count. 
--					
-- Rev History: 3-31-2014:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use IEEE.math_real.all;

Library UNISIM; 
use UNISIM.Vcomponents.all; 

use work.model_specific_package.all;
use work.tb_package.all;
use work.transceiver_package.all;

entity synthetic_sin_wave is
port (
    
    -- Clock and reset generated by the PLDA EZDMA core.
    clk				: in std_logic;
    rst				: in std_logic;
    
    wave_start		: in std_logic;
    
    synth_wave		: out std_logic_vector((RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0)  
);
end synthetic_sin_wave;

architecture structural of synthetic_sin_wave is
	constant FSDIVISOR			: real := 4.0;
	
	signal wave_start_r		: std_logic;
	signal phase_counter	: real;
    signal synth_sine_wave	: real;  
    signal synth_sine_wave_R: real;  
    signal sine_wave		: std_logic_vector(RX_ITEM_SIZE-1 downto 0);  
    signal sine_wave_R		: std_logic_vector(RX_ITEM_SIZE-1 downto 0);  

begin   

	M376 : if MODEL = x"376" generate
	process (clk,rst)
	begin
		if rst = '1' then                      
			phase_counter <= 0.0;
			synth_sine_wave <= 0.0;
			synth_sine_wave_R <= 0.0;
			wave_start_r <= '0';
			synth_wave <= (others => '0');
		elsif rising_edge (clk) then
			wave_start_r <= wave_start;
			if(wave_start_r = '1') then
				phase_counter <= phase_counter + 2.0;        
				synth_sine_wave <= ((2.0 ** (real(RX_ITEM_SIZE)-1.0))-1.0) * sin(2.0*math_pi*phase_counter/FSDIVISOR);
				synth_sine_wave_R <= ((2.0 ** (real(RX_ITEM_SIZE)-1.0))-1.0) * sin(2.0*math_pi*(phase_counter + 1.0)/FSDIVISOR);
				synth_wave <= std_logic_vector(to_signed(integer(synth_sine_wave_R),RX_ITEM_SIZE/RX_INTERLEAVE)) & std_logic_vector(to_signed(integer(synth_sine_wave),RX_ITEM_SIZE/RX_INTERLEAVE));
			end if;
		elsif falling_edge (clk) then
			if(wave_start_r = '1') then
				phase_counter <= phase_counter + 2.0;        
				synth_sine_wave <= ((2.0 ** (real(RX_ITEM_SIZE)-1.0))-1.0) * sin(2.0*math_pi*phase_counter/FSDIVISOR);
				synth_sine_wave_R <= ((2.0 ** (real(RX_ITEM_SIZE)-1.0))-1.0) * sin(2.0*math_pi*(phase_counter + 1.0)/FSDIVISOR);
				synth_wave <= std_logic_vector(to_signed(integer(synth_sine_wave_R),RX_ITEM_SIZE/RX_INTERLEAVE)) & std_logic_vector(to_signed(integer(synth_sine_wave),RX_ITEM_SIZE/RX_INTERLEAVE));
			end if;
	   end if;                                   
	end process;                               
	end generate;
	
	M377 : if MODEL = x"377" generate
	process (clk,rst)
	begin
		if rst = '1' then                      
			phase_counter <= 0.0;
			wave_start_r <= '0';
			sine_wave <= (others => '0');
			synth_wave <= (others => '0');
		elsif rising_edge (clk) then
			wave_start_r <= wave_start;
			if(wave_start_r = '1') then
				phase_counter <= phase_counter + 1.0;        
				sine_wave <= std_logic_vector(to_signed(integer(((2.0 ** (real(RX_ITEM_SIZE)-1.0))-1.0) * sin(2.0*math_pi*phase_counter/FSDIVISOR)),RX_ITEM_SIZE));
				synth_wave(0) 	<= sine_wave(1);
				synth_wave(1) 	<= sine_wave(3);
				synth_wave(2) 	<= sine_wave(5);
				synth_wave(3) 	<= sine_wave(7);
				synth_wave(4)	<= sine_wave(9);
				synth_wave(5) 	<= sine_wave(11);
				synth_wave(6) 	<= sine_wave(13);   
				synth_wave(7) 	<= sine_wave(15);
			end if;                                        
		elsif falling_edge (clk) then
			if(wave_start_r = '1') then
				synth_wave(0) <= sine_wave(0);
				synth_wave(1) <= sine_wave(2);
				synth_wave(2) <= sine_wave(4);
				synth_wave(3) <= sine_wave(6);
				synth_wave(4) <= sine_wave(8); 
				synth_wave(5) <= sine_wave(10);            
				synth_wave(6) <= sine_wave(12);
				synth_wave(7) <= sine_wave(14);
			end if;
	   end if;                                   
	end process;
	end generate;

end structural;
